#[non_exhaustive]pub struct GeodesicDirect {
pub latitude_rad: f64,
pub longitude_rad: f64,
pub final_azimuth_rad: f64,
}Expand description
The end of a geodesic from a start, an azimuth and a distance: Karney’s direct problem.
The end has no height: the geodesic lies on the ellipsoid’s surface. GeodesicDirect::end
makes a position of it at a height the caller chooses.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.latitude_rad: f64The end’s geodetic latitude, rad, in [−π/2, π/2].
longitude_rad: f64The end’s longitude, rad, in [−π, π].
final_azimuth_rad: f64Azimuth α₂ arriving at the end, rad, clockwise from north, in [−π, π].
Implementations
Sourceimpl GeodesicDirect
impl GeodesicDirect
Sourcepub fn end(&self, height_m: f64) -> Result<Geodetic, CoreError>
pub fn end(&self, height_m: f64) -> Result<Geodetic, CoreError>
The end as a position at ellipsoidal height height_m.
Errors
As Geodetic::new: CoreError::Domain if the height is not finite.