ecef_from_enu_rotation

Function ecef_from_enu_rotation 

pub fn ecef_from_enu_rotation(point: Geodetic) -> DMat3
The rotation whose columns are the East, North and Up unit vectors at point, resolved in ECEF. It maps ENU components to ECEF components; its transpose maps back.

ê = (−sin λ, cos λ, 0)
n̂ = (−sin φ cos λ, −sin φ sin λ, cos φ)
û = (cos φ cos λ, cos φ sin λ, sin φ)