DMat3

Struct DMat3 

#[repr(C)]
pub struct DMat3 { pub x_axis: DVec3, pub y_axis: DVec3, pub z_axis: DVec3, }
A 3x3 column major matrix.

This 3x3 matrix type features convenience methods for creating and using linear and affine transformations. If you are primarily dealing with 2D affine transformations the DAffine2 type is much faster and more space efficient than using a 3x3 matrix.

Linear transformations including 3D rotation and scale can be created using methods such as Self::from_diagonal(), Self::from_quat(), Self::from_axis_angle(), Self::from_rotation_x(), Self::from_rotation_y(), or Self::from_rotation_z().

The resulting matrices can be use to transform 3D vectors using regular vector multiplication.

Affine transformations including 2D translation, rotation and scale can be created using methods such as Self::from_translation(), Self::from_angle(), Self::from_scale() and Self::from_scale_angle_translation().

The Self::transform_point2() and Self::transform_vector2() convenience methods are provided for performing affine transforms on 2D vectors and points. These multiply 2D inputs as 3D vectors with an implicit z value of 1 for points and 0 for vectors respectively. These methods assume that Self contains a valid affine transform.

Fields

x_axis: DVec3y_axis: DVec3z_axis: DVec3

Implementations



impl DMat3

pub const ZERO: DMat3

A 3x3 matrix with all elements set to 0.0.

pub const IDENTITY: DMat3

A 3x3 identity matrix, where all diagonal elements are 1, and all off-diagonal elements are 0.

pub const NAN: DMat3

All NAN:s.

pub const fn from_cols(x_axis: DVec3, y_axis: DVec3, z_axis: DVec3) -> DMat3

Creates a 3x3 matrix from three column vectors.

pub const fn from_cols_array(m: &[f64; 9]) -> DMat3

Creates a 3x3 matrix from a [f64; 9] array stored in column major order. If your data is stored in row major you will need to transpose the returned matrix.

pub const fn to_cols_array(&self) -> [f64; 9]

Creates a [f64; 9] array storing data in column major order. If you require data in row major order transpose the matrix first.

pub const fn from_cols_array_2d(m: &[[f64; 3]; 3]) -> DMat3

Creates a 3x3 matrix from a [[f64; 3]; 3] 3D array stored in column major order. If your data is in row major order you will need to transpose the returned matrix.

pub const fn to_cols_array_2d(&self) -> [[f64; 3]; 3]

Creates a [[f64; 3]; 3] 3D array storing data in column major order. If you require data in row major order transpose the matrix first.

pub const fn from_diagonal(diagonal: DVec3) -> DMat3

Creates a 3x3 matrix with its diagonal set to diagonal and all other entries set to 0.

pub fn from_mat4(m: DMat4) -> DMat3

Creates a 3x3 matrix from a 4x4 matrix, discarding the 4th row and column.

pub fn from_mat4_minor(m: DMat4, i: usize, j: usize) -> DMat3

Creates a 3x3 matrix from the minor of the given 4x4 matrix, discarding the ith column and jth row.

Panics

Panics if i or j is greater than 3.

pub fn from_quat(rotation: DQuat) -> DMat3

Creates a 3D rotation matrix from the given quaternion.

Panics

Will panic if rotation is not normalized when glam_assert is enabled.

pub fn from_axis_angle(axis: DVec3, angle: f64) -> DMat3

Creates a 3D rotation matrix from a normalized rotation axis and angle (in radians).

Panics

Will panic if axis is not normalized when glam_assert is enabled.

pub fn from_euler(order: EulerRot, a: f64, b: f64, c: f64) -> DMat3

Creates a 3D rotation matrix from the given euler rotation sequence and the angles (in radians).

pub fn to_euler(&self, order: EulerRot) -> (f64, f64, f64)

Extract Euler angles with the given Euler rotation order.

Note if the input matrix contains scales, shears, or other non-rotation transformations then the resulting Euler angles will be ill-defined.

Panics

Will panic if any input matrix column is not normalized when glam_assert is enabled.

pub fn from_rotation_x(angle: f64) -> DMat3

Creates a 3D rotation matrix from angle (in radians) around the x axis.

pub fn from_rotation_y(angle: f64) -> DMat3

Creates a 3D rotation matrix from angle (in radians) around the y axis.

pub fn from_rotation_z(angle: f64) -> DMat3

Creates a 3D rotation matrix from angle (in radians) around the z axis.

pub fn from_translation(translation: DVec2) -> DMat3

Creates an affine transformation matrix from the given 2D translation.

The resulting matrix can be used to transform 2D points and vectors. See Self::transform_point2() and Self::transform_vector2().

pub fn from_angle(angle: f64) -> DMat3

Creates an affine transformation matrix from the given 2D rotation angle (in radians).

The resulting matrix can be used to transform 2D points and vectors. See Self::transform_point2() and Self::transform_vector2().

pub fn from_scale_angle_translation( scale: DVec2, angle: f64, translation: DVec2, ) -> DMat3

Creates an affine transformation matrix from the given 2D scale, rotation angle (in radians) and translation.

The resulting matrix can be used to transform 2D points and vectors. See Self::transform_point2() and Self::transform_vector2().

pub fn from_scale(scale: DVec2) -> DMat3

Creates an affine transformation matrix from the given non-uniform 2D scale.

The resulting matrix can be used to transform 2D points and vectors. See Self::transform_point2() and Self::transform_vector2().

Panics

Will panic if all elements of scale are zero when glam_assert is enabled.

pub fn from_mat2(m: DMat2) -> DMat3

Creates an affine transformation matrix from the given 2x2 matrix.

The resulting matrix can be used to transform 2D points and vectors. See Self::transform_point2() and Self::transform_vector2().

pub const fn from_cols_slice(slice: &[f64]) -> DMat3

Creates a 3x3 matrix from the first 9 values in slice.

Panics

Panics if slice is less than 9 elements long.

pub fn write_cols_to_slice(&self, slice: &mut [f64])

Writes the columns of self to the first 9 elements in slice.

Panics

Panics if slice is less than 9 elements long.

pub fn col(&self, index: usize) -> DVec3

Returns the matrix column for the given index.

Panics

Panics if index is greater than 2.

pub fn col_mut(&mut self, index: usize) -> &mut DVec3

Returns a mutable reference to the matrix column for the given index.

Panics

Panics if index is greater than 2.

pub fn row(&self, index: usize) -> DVec3

Returns the matrix row for the given index.

Panics

Panics if index is greater than 2.

pub fn is_finite(&self) -> bool

Returns true if, and only if, all elements are finite. If any element is either NaN, positive or negative infinity, this will return false.

pub fn is_nan(&self) -> bool

Returns true if any elements are NaN.

pub fn transpose(&self) -> DMat3

Returns the transpose of self.

pub fn diagonal(&self) -> DVec3

Returns the diagonal of self.

pub fn determinant(&self) -> f64

Returns the determinant of self.

pub fn inverse(&self) -> DMat3

Returns the inverse of self.

If the matrix is not invertible the returned matrix will be invalid.

Panics

Will panic if the determinant of self is zero when glam_assert is enabled.

pub fn try_inverse(&self) -> Option<DMat3>

Returns the inverse of self or None if the matrix is not invertible.

pub fn inverse_or_zero(&self) -> DMat3

Returns the inverse of self or DMat3::ZERO if the matrix is not invertible.

pub fn transform_point2(&self, rhs: DVec2) -> DVec2

Transforms the given 2D vector as a point.

This is the equivalent of multiplying rhs as a 3D vector where z is 1.

This method assumes that self contains a valid affine transform.

Panics

Will panic if the 2nd row of self is not (0, 0, 1) when glam_assert is enabled.

pub fn transform_vector2(&self, rhs: DVec2) -> DVec2

Rotates the given 2D vector.

This is the equivalent of multiplying rhs as a 3D vector where z is 0.

This method assumes that self contains a valid affine transform.

Panics

Will panic if the 2nd row of self is not (0, 0, 1) when glam_assert is enabled.

pub fn look_to_lh(dir: DVec3, up: DVec3) -> DMat3

👎Deprecated since 0.33.1:

use the glam::dcamera::lh::view::look_to_mat3 function instead

Creates a left-handed view matrix using a facing direction and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=forward.

Panics

Will panic if dir or up are not normalized when glam_assert is enabled.

pub fn look_to_rh(dir: DVec3, up: DVec3) -> DMat3

👎Deprecated since 0.33.1:

use the glam::dcamera::rh::view::look_to_mat3 function instead

Creates a right-handed view matrix using a facing direction and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=back.

Panics

Will panic if dir or up are not normalized when glam_assert is enabled.

pub fn look_at_lh(eye: DVec3, center: DVec3, up: DVec3) -> DMat3

👎Deprecated since 0.33.1:

use the glam::dcamera::lh::view::look_at_mat3 function instead

Creates a left-handed view matrix using a camera position, a focal point and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=forward.

Panics

Will panic if up is not normalized when glam_assert is enabled.

pub fn look_at_rh(eye: DVec3, center: DVec3, up: DVec3) -> DMat3

👎Deprecated since 0.33.1:

use the glam::dcamera::rh::view::look_at_mat3 function instead

Creates a right-handed view matrix using a camera position, a focal point and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=back.

Panics

Will panic if up is not normalized when glam_assert is enabled.

pub fn mul_vec3(&self, rhs: DVec3) -> DVec3

Transforms a 3D vector.

pub fn mul_transpose_vec3(&self, rhs: DVec3) -> DVec3

Transforms a 3D vector by the transpose of self.

pub fn mul_mat3(&self, rhs: &DMat3) -> DMat3

Multiplies two 3x3 matrices.

pub fn add_mat3(&self, rhs: &DMat3) -> DMat3

Adds two 3x3 matrices.

pub fn sub_mat3(&self, rhs: &DMat3) -> DMat3

Subtracts two 3x3 matrices.

pub fn mul_scalar(&self, rhs: f64) -> DMat3

Multiplies a 3x3 matrix by a scalar.

pub fn mul_diagonal_scale(&self, scale: DVec3) -> DMat3

Multiply self by a scaling vector scale. This is faster than creating a whole diagonal scaling matrix and then multiplying that. This operation is commutative.

pub fn div_scalar(&self, rhs: f64) -> DMat3

Divides a 3x3 matrix by a scalar.

pub fn recip(&self) -> DMat3

Returns a matrix containing the reciprocal 1.0/n of each element of self.

pub fn abs_diff_eq(&self, rhs: DMat3, max_abs_diff: f64) -> bool

Returns true if the absolute difference of all elements between self and rhs is less than or equal to max_abs_diff.

This can be used to compare if two matrices contain similar elements. It works best when comparing with a known value. The max_abs_diff that should be used used depends on the values being compared against.

For more see comparing floating point numbers.

pub fn abs(&self) -> DMat3

Takes the absolute value of each element in self

pub fn as_mat3(&self) -> Mat3