DQuat

Struct DQuat 

#[repr(C)]
pub struct DQuat { pub x: f64, pub y: f64, pub z: f64, pub w: f64, }
A quaternion representing an orientation.

This quaternion is intended to be of unit length but may denormalize due to floating point “error creep” which can occur when successive quaternion operations are applied.

Fields

x: f64y: f64z: f64w: f64

Implementations



impl DQuat

pub const IDENTITY: DQuat

The identity quaternion. Corresponds to no rotation.

pub const NAN: DQuat

All NANs.

pub const fn from_xyzw(x: f64, y: f64, z: f64, w: f64) -> DQuat

Creates a new rotation quaternion.

This should generally not be called manually unless you know what you are doing. Use one of the other constructors instead such as identity or from_axis_angle.

from_xyzw is mostly used by unit tests and serde deserialization.

Preconditions

This function does not check if the input is normalized, it is up to the user to provide normalized input or to normalized the resulting quaternion.

pub const fn from_array(a: [f64; 4]) -> DQuat

Creates a rotation quaternion from an array.

Preconditions

This function does not check if the input is normalized, it is up to the user to provide normalized input or to normalized the resulting quaternion.

pub const fn from_vec4(v: DVec4) -> DQuat

Creates a new rotation quaternion from a 4D vector.

Preconditions

This function does not check if the input is normalized, it is up to the user to provide normalized input or to normalized the resulting quaternion.

pub fn from_slice(slice: &[f64]) -> DQuat

Creates a rotation quaternion from a slice.

Preconditions

This function does not check if the input is normalized, it is up to the user to provide normalized input or to normalized the resulting quaternion.

Panics

Panics if slice length is less than 4.

pub fn write_to_slice(self, slice: &mut [f64])

Writes the quaternion to an unaligned slice.

Panics

Panics if slice length is less than 4.

pub fn from_axis_angle(axis: DVec3, angle: f64) -> DQuat

Create a quaternion for a normalized rotation axis and angle (in radians).

The axis must be a unit vector.

Panics

Will panic if axis is not normalized when glam_assert is enabled.

pub fn from_scaled_axis(v: DVec3) -> DQuat

Create a quaternion that rotates v.length() radians around v.normalize().

from_scaled_axis(Vec3::ZERO) results in the identity quaternion.

pub fn from_rotation_x(angle: f64) -> DQuat

Creates a quaternion from the angle (in radians) around the x axis.

pub fn from_rotation_y(angle: f64) -> DQuat

Creates a quaternion from the angle (in radians) around the y axis.

pub fn from_rotation_z(angle: f64) -> DQuat

Creates a quaternion from the angle (in radians) around the z axis.

pub fn from_euler(euler: EulerRot, a: f64, b: f64, c: f64) -> DQuat

Creates a quaternion from the given Euler rotation sequence and the angles (in radians).

pub fn from_rotation_axes(x_axis: DVec3, y_axis: DVec3, z_axis: DVec3) -> DQuat

From the columns of a 3x3 rotation matrix.

Note if the input axes contain scales, shears, or other non-rotation transformations then the output of this function is ill-defined.

Panics

Will panic if any axis is not normalized when glam_assert is enabled.

pub fn from_mat3(mat: &DMat3) -> DQuat

Creates a quaternion from a 3x3 rotation matrix.

Note if the input matrix contain scales, shears, or other non-rotation transformations then the resulting quaternion will be ill-defined.

Panics

Will panic if any input matrix column is not normalized when glam_assert is enabled.

pub fn from_mat4(mat: &DMat4) -> DQuat

Creates a quaternion from the upper 3x3 rotation matrix inside a homogeneous 4x4 matrix.

Note if the upper 3x3 matrix contain scales, shears, or other non-rotation transformations then the resulting quaternion will be ill-defined.

Panics

Will panic if any column of the upper 3x3 rotation matrix is not normalized when glam_assert is enabled.

pub fn from_rotation_arc(from: DVec3, to: DVec3) -> DQuat

Gets the minimal rotation for transforming from to to. The rotation is in the plane spanned by the two vectors. Will rotate at most 180 degrees.

The inputs must be unit vectors.

from_rotation_arc(from, to) * from ≈ to.

For near-singular cases (from≈to and from≈-to) the current implementation is only accurate to about 0.001 (for f32).

Panics

Will panic if from or to are not normalized when glam_assert is enabled.

pub fn from_rotation_arc_colinear(from: DVec3, to: DVec3) -> DQuat

Gets the minimal rotation for transforming from to either to or -to. This means that the resulting quaternion will rotate from so that it is colinear with to.

The rotation is in the plane spanned by the two vectors. Will rotate at most 90 degrees.

The inputs must be unit vectors.

to.dot(from_rotation_arc_colinear(from, to) * from).abs() ≈ 1.

Panics

Will panic if from or to are not normalized when glam_assert is enabled.

pub fn from_rotation_arc_2d(from: DVec2, to: DVec2) -> DQuat

Gets the minimal rotation for transforming from to to. The resulting rotation is around the z axis. Will rotate at most 180 degrees.

The inputs must be unit vectors.

from_rotation_arc_2d(from, to) * from ≈ to.

For near-singular cases (from≈to and from≈-to) the current implementation is only accurate to about 0.001 (for f32).

Panics

Will panic if from or to are not normalized when glam_assert is enabled.

pub fn look_to_lh(dir: DVec3, up: DVec3) -> DQuat

👎Deprecated since 0.33.1:

use the glam::dcamera::lh::view::look_to_quat function instead

Creates a quaterion rotation from a facing direction and an up direction.

For a left-handed view coordinate system with +X=right, +Y=up and +Z=forward.

Panics

Will panic if up is not normalized when glam_assert is enabled.

pub fn look_to_rh(dir: DVec3, up: DVec3) -> DQuat

👎Deprecated since 0.33.1:

use the glam::dcamera::rh::view::look_to_quat function instead

Creates a quaterion rotation from facing direction and an up direction.

For a right-handed view coordinate system with +X=right, +Y=up and +Z=back.

Panics

Will panic if dir and up are not normalized when glam_assert is enabled.

pub fn look_at_lh(eye: DVec3, center: DVec3, up: DVec3) -> DQuat

👎Deprecated since 0.33.1:

use the glam::dcamera::lh::view::look_at_quat function instead

Creates a quaternion rotation from a camera position, a focal point, and an up direction.

For a left-handed view coordinate system with +X=right, +Y=up and +Z=forward.

Panics

Will panic if up is not normalized when glam_assert is enabled.

pub fn look_at_rh(eye: DVec3, center: DVec3, up: DVec3) -> DQuat

👎Deprecated since 0.33.1:

use the glam::dcamera::rh::view::look_at_quat function instead

Creates a quaternion rotation using a camera position, an up direction, and a focal point.

For a right-handed view coordinate system with +X=right, +Y=up and +Z=back.

Panics

Will panic if up is not normalized when glam_assert is enabled.

pub fn to_axis_angle(self) -> (DVec3, f64)

Returns the rotation axis (normalized) and angle (in radians) of self.

pub fn to_scaled_axis(self) -> DVec3

Returns the rotation axis scaled by the rotation in radians.

pub fn to_euler(self, order: EulerRot) -> (f64, f64, f64)

Returns the rotation angles for the given euler rotation sequence.

pub const fn to_array(&self) -> [f64; 4]

Converts self to [x, y, z, w]

pub fn xyz(self) -> DVec3

Returns the vector part of the quaternion.

pub fn conjugate(self) -> DQuat

Returns the quaternion conjugate of self. For a unit quaternion the conjugate is also the inverse.

pub fn inverse(self) -> DQuat

Returns the inverse of a normalized quaternion.

Typically quaternion inverse returns the conjugate of a normalized quaternion. Because self is assumed to already be unit length this method does not normalize before returning the conjugate.

Panics

Will panic if self is not normalized when glam_assert is enabled.

pub fn dot(self, rhs: DQuat) -> f64

Computes the dot product of self and rhs. The dot product is equal to the cosine of the angle between two quaternion rotations.

pub fn length(self) -> f64

Computes the length of self.

pub fn length_squared(self) -> f64

Computes the squared length of self.

This is generally faster than length() as it avoids a square root operation.

pub fn length_recip(self) -> f64

Computes 1.0 / length().

For valid results, self must not be of length zero.

pub fn normalize(self) -> DQuat

Returns self normalized to length 1.0.

For valid results, self must not be of length zero.

Panics

Will panic if self is zero length when glam_assert is enabled.

pub fn is_finite(self) -> bool

Returns true if, and only if, all elements are finite. If any element is either NaN, positive or negative infinity, this will return false.

pub fn is_nan(self) -> bool

Returns true if any elements are NAN.

pub fn is_normalized(self) -> bool

Returns whether self of length 1.0 or not.

Uses a precision threshold of 1e-6.

pub fn is_near_identity(self) -> bool

Returns true if self represents a rotation near the identity.

pub fn angle_between(self, rhs: DQuat) -> f64

Returns the angle (in radians) for the minimal rotation between two quaternions in the range [0, +π].

Both quaternions must be normalized.

Panics

Will panic if self or rhs are not normalized when glam_assert is enabled.

pub fn rotate_towards(self, rhs: DQuat, max_angle: f64) -> DQuat

Rotates towards rhs up to max_angle (in radians).

When max_angle is 0.0, the result will be equal to self. When max_angle is equal to self.angle_between(rhs), the result will be equal to rhs. If max_angle is negative, rotates towards the exact opposite of rhs. Will not go past the target.

Both quaternions must be normalized.

Panics

Will panic if self or rhs are not normalized when glam_assert is enabled.

pub fn abs_diff_eq(self, rhs: DQuat, max_abs_diff: f64) -> bool

Returns true if the absolute difference of all elements between self and rhs is less than or equal to max_abs_diff.

This can be used to compare if two quaternions contain similar elements. It works best when comparing with a known value. The max_abs_diff that should be used used depends on the values being compared against.

For more see comparing floating point numbers.

pub fn lerp(self, end: DQuat, s: f64) -> DQuat

Performs a linear interpolation between self and rhs based on the value s.

When s is 0.0, the result will be equal to self. When s is 1.0, the result will be equal to rhs.

Panics

Will panic if self or end are not normalized when glam_assert is enabled.

pub fn slerp(self, end: DQuat, s: f64) -> DQuat

Performs a spherical linear interpolation between self and end based on the value s.

When s is 0.0, the result will be equal to self. When s is 1.0, the result will be equal to end.

Panics

Will panic if self or end are not normalized when glam_assert is enabled.

pub fn slerp_long(self, end: DQuat, s: f64) -> DQuat

Performs a spherical linear interpolation between self and end based on the value s, preserving the rotation direction.

When s is 0.0, the result will be equal to self. When s is 1.0, the result will be equal to end.

When the dot product of self and end is negative, the standard slerp will flip the end quaternion to take the shortest path, while this method will take the longer arc. This is useful when the intended rotation direction must be preserved.

Panics

Will panic if self or end are not normalized when glam_assert is enabled.

pub fn mul_vec3(self, rhs: DVec3) -> DVec3

Multiplies a quaternion and a 3D vector, returning the rotated vector.

Panics

Will panic if self is not normalized when glam_assert is enabled.

pub fn mul_quat(self, rhs: DQuat) -> DQuat

Multiplies two quaternions. If they each represent a rotation, the result will represent the combined rotation.

Note that due to floating point rounding the result may not be perfectly normalized.

Panics

Will panic if self or rhs are not normalized when glam_assert is enabled.

pub fn from_affine3(a: &DAffine3) -> DQuat

Creates a quaternion from a 3x3 rotation matrix inside a 3D affine transform.

Note if the input affine matrix contain scales, shears, or other non-rotation transformations then the resulting quaternion will be ill-defined.

Panics

Will panic if any input affine matrix column is not normalized when glam_assert is enabled.

pub fn as_quat(self) -> Quat